<?php

namespace Concrete\Package\Chatwee\Src;

use Concrete\Core\Support\Facade\Events;
use User;
use UserInfo;
use Package;
use Exception;
use View;
use Page;
use Log;
use Database;
use Chatwee\Sso\SsoConfiguration;
use Chatwee\Sso\SsoManager;
use Chatwee\Sso\SsoSession;
use Chatwee\Sso\SsoUser;

defined('C5_EXECUTE') or die("Access Denied.");

class ChatweeEvents
{
	private $config;

	public function __construct()
	{

		$pkg = Package::getByHandle("chatwee");
		$this->config = $pkg->getConfig()->get("concrete.chatwee");
	}

	public function onUserLoginEventcallback($event)
	{
	    if ($this->config["chatwee_sso_enable"] === "1")
	    {
	    	$this->setChatweeSsoCredentials();
            $this->ssoLogin();
	    }
	}

	public function onUserLogoutEventCallback()
	{
	    if ($this->config["chatwee_sso_enable"] === "1")
	    {
	    	$this->setChatweeSsoCredentials();
            $this->ssoLogout();
	    }
	}

	public function onBeforeRenderEventCallback()
	{
		$currentPage = Page::getCurrentPage();

    	if($this->config["chatwee_code"] && $currentPage->isEditMode() === false && $currentPage->isAdminArea() === false)
    	{
    		$chatwee_code = html_entity_decode($this->config["chatwee_code"]);
			$view = new View;
			$view->addFooterItem($chatwee_code);
		}

		if ($this->config["chatwee_sso_enable"] === "1")
		{
			$this->setChatweeSsoCredentials();
			$this->synchronizeUserSessions();
		}
	}

	public function setChatweeSsoCredentials()
	{
        SsoConfiguration::setChatId($this->config["chatwee_sso_chat_id"]);
        SsoConfiguration::setClientKey($this->config["chatwee_sso_api_key"]);
        SsoConfiguration::setClientKey($this->config["chatwee_sso_api_key"]);
	}

	public function synchronizeUserSessions()
	{
		$user = new User();
    	$isUserLoggedIn = $user->isLoggedIn();

		if($isUserLoggedIn === true && SsoSession::isSessionSet() === false)
		{
			$this->ssoLogin();
		}
		if($isUserLoggedIn === true && SsoSession::isSessionSet() === true)
		{
			try
			{
				$isSessionValid = SsoUser::validateSession(array(
					'sessionId' => SsoSession::getSessionId(),
				));

				if($isSessionValid === false)
				{
					$this->ssoLogin();
				}
			}
			catch(Exception $exception)
			{
				Log::error("Error while reinitializng SSO session: " . $exception->getMessage());
			}
		}
	}

    public function ssoLogin()
    {
		$user = new User();
    	$userId = $user->getUserID();

        $chatweeUserId = $this->getChatweeUserId($userId);

        if(!$chatweeUserId)
        {
            $chatweeUserId = $this->ssoRegister();
        }

        try
        {
            SsoManager::editUser(array(
                'userId' => $chatweeUserId,
                'login'  => $user->getUserName(),
                'avatar' => $this->getUserAvatar($userId),
                'isAdmin' => $this->isChatweeAdmin($user),
            ));
        }
        catch(Exception $exception)
        {
			if(strpos($exception->getMessage(), "User does not exist") !== false) {
				$chatweeUserId = $this->ssoRegister();
			} else {
				Log::error("Error while executing sso_login: " . $exception->getMessage());
			}
        }

        try
        {
            $sessionId = SsoManager::loginUser(array(
                'userId' => $chatweeUserId,
            ));
        }
        catch(Exception $exception)
        {
        	Log::error("Error while executing sso_login: " . $exception->getMessage());
        }
    }

    public function ssoLogout()
    {
        try
        {
            SsoManager::logoutUser();
        }
        catch(Exception $exception)
        {
        	log::error("Error while executing sso_logout: " . $exception->getMessage());
            return false;
        }
    }

    public function ssoRegister()
    {
		$user = new User();
    	$userId = $user->getUserID();

        try
        {
            $chatweeUserId = SsoManager::registerUser(array(
                'login' => $user->getUserName(),
                'avatar' => $this->getUserAvatar($userId),
                'isAdmin' => $this->isChatweeAdmin($user),
            ));

            $this->assignChatweeUserId($userId, $chatweeUserId);

            return $chatweeUserId;
        }
        catch(Exception $exception)
        {
        	log::error("Error while executing sso_register: " . $exception->getMessage());
            return null;
        }
    }

	public function assignChatweeUserId($userId, $chatweeUserId)
	{
		$db = Database::connection();

		$db->Execute("DELETE FROM chatweeUsers WHERE userId=?", array($userId));
        $db->Execute("INSERT INTO chatweeUsers VALUES(?, ?)", array($userId, $chatweeUserId));
	}

	public function getChatweeUserId($userId)
	{
		$db = Database::connection();

        $data = $db->GetRow("select chatweeUserId from chatweeUsers where userId='" . $userId . "'");

        if(!$data)
        {
        	return null;
        }

        return $data["chatweeUserId"];
	}

	public function getUserAvatar($userId)
	{
		$userInfo = UserInfo::getByID($userId);
		return $userInfo->hasAvatar() ? $userInfo->getUserAvatar()->getPath() : "";
	}

	public function isChatweeAdmin($user)
	{
		if($user->isSuperUser() === true)
		{
			return true;
		}

		$adminGroups = $this->config["chatwee_sso_admin_groups"];
		if(!is_array($adminGroups) || empty($adminGroups))
		{
			return false;
		}

		foreach ($user->getUserGroups() as $userGroup)
		{
			if (in_array($userGroup, $adminGroups))
			{

				return true;
			}
		}

		return false;
	}
}